// =========================================================
// Copyright (c) Apache Project 2000.
// Time-stamp: <08 Nov 00 18:41:02 Jean-Guillaume.Dumas@imag.fr> 
// Modified from :

/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
//******************************************************************
// IntegrationDisplay.java - A class that handles the data display
//                           portion of the Integration applet.
//
// Author:  George Stebbins
//
//******************************************************************
 
package Integrate;

import java.awt.*;
import hamlet.*; 


//******************************************************************
//******************************************************************
//
// IntegrationDisplay class
//
//******************************************************************
//******************************************************************

class IntegrationDisplay extends Panel {

    private NumericLabel actual;
    private NumericLabel calculated;
    private NumericLabel difference;

    private Label actualTitle;
    private Label calculatedTitle;
    private Label differenceTitle;

    private NumericLabel percent;
    private Panel percents;

    private PieChart pie;
    private PieChart pie_cent;

    private final static Color actualColor = ColorList.RED;
    private final static Color centColor = ColorList.BLUE;

    private final static Color diffColor = ColorList.DARK_GRAY;

    
    //----------------------------------------------------
    // Constructor
    //----------------------------------------------------
    
    IntegrationDisplay () {

	Panel myPanel = new Panel();
	
	// Build the three titles
	
	Panel titles = new Panel();
	actualTitle = new Label("Actual Value", Label.RIGHT);
	calculatedTitle = new Label("Calculated Value", Label.RIGHT);
	differenceTitle = new Label("Difference", Label.RIGHT);
	
	titles.add(actualTitle);
	titles.add(calculatedTitle);
	titles.add(differenceTitle);
	titles.setLayout(new GridLayout(3,1,0,0));

	
	// Build the three output values
	
	Panel values = new Panel();
	actual = new NumericLabel(0.0, Label.CENTER);
	calculated = new NumericLabel(0.0, Label.CENTER);
	difference = new NumericLabel(0.0, Label.CENTER);
	values.add(actual);
	values.add(calculated);
	values.add(difference);
	values.setLayout(new GridLayout (3,1,0,0));


	// Build the % difference panel
	
	percents = new Panel();
	percents.add(new Label("% difference", Label.CENTER));
	percent = new NumericLabel(0.0, Label.CENTER);
	percents.add(percent);
	percents.setLayout(new GridLayout(2,1,0,0));


	// Build the pie chart panel
	
	pie = new PieChart(30, 80, actualColor, diffColor);
	pie_cent = new PieChart(30, 80, centColor, diffColor);

	
	// Put the panels into the superPanel
	
	myPanel.setLayout(new GridLayout(1,4,0,0));

	myPanel.add(titles);
	myPanel.add(values);
	myPanel.add(percents);
	myPanel.add(pie);
	myPanel.add(pie_cent);


	// Add the main panel
	
	add(myPanel);

    }
    
    
    //----------------------------------------------------
    // Update the display
    //----------------------------------------------------

    public void doUpdate (double actualValue, double calcValue) {

	double   diff;
	double   maxValue;
	double   maxPercent;
	double   diffPercent;
	

	// Calculate the data needed
	
	diff = Math.abs(actualValue - calcValue);

	if (Math.abs(actualValue) > Math.abs(calcValue)) {
	    maxValue = actualValue;
	} else {
	    maxValue = calcValue;
	}

	if (actualValue == 0) {
	    if (diff == 0)
		diffPercent = 0.0;
	    else
// 		diffPercent = 100.0;
                diffPercent = calcValue;
	} else {
	    diffPercent = Math.abs(100.0 * diff / actualValue);
	}
	
	// Fill the values in the text fields
	
	actual.setNumber(actualValue);
	calculated.setNumber(calcValue);
	difference.setNumber(diff);
	percent.setNumber(diffPercent);
	

	// Update the pie chart

	pie.reset (diffPercent);
	if (diffPercent > 1) {
            pie_cent.show(false);
            pie_cent.reset( 0.0 );
        } else {
            pie_cent.reset ((diffPercent-(int)diffPercent)*100);
            pie_cent.show(true);
        }
    }


    //----------------------------------------------------
    // Hide all components that contain calculated values
    //----------------------------------------------------

    public void doShow (boolean b) {
	calculatedTitle.show(b);
	differenceTitle.show(b);
	calculated.show(b);
	difference.show(b);
	percent.show(b);
	percents.show(b);
	pie.show(b);
 	pie_cent.show( (b && (pie_cent.get_percent()>0) ) );
   }

}



