// =========================================================
// Copyright (c) Apache Project 2000.
// Time-stamp: <08 Nov 00 18:38:35 Jean-Guillaume.Dumas@imag.fr> 
// Modified from :

/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

//******************************************************************
// PieChart.java
//
// Author:  George Stebbins
//
//******************************************************************
 
package Integrate;

//----------------------------------------------------
// These are the needed inclusions
//----------------------------------------------------
 
import java.awt.*;
 


//******************************************************************
//******************************************************************
//
// PieChart class
//
//******************************************************************
//******************************************************************

class PieChart extends Canvas {

    //----------------------------------------------------
    // Member data
    //----------------------------------------------------
    
    private int       theRadius;
    private double    thePct;
    private double    theOldPct;
    private Color     theColor1;
    private Color     theColor2;

    
    //----------------------------------------------------
    // Constructor
    //----------------------------------------------------

    public PieChart (int radius, double percentage,
		     Color  color1, Color  color2)     {

       theRadius = radius;
       thePct = percentage;
       if (thePct > 100) thePct = 100;
       theColor1 = color1;
       theColor2 = color2;
       resize(2*radius, 2*radius);
    }

    
    //----------------------------------------------------
    // Change the percentage that is displayed
    //----------------------------------------------------

    public void reset (double percentage) {
	theOldPct = thePct;
	thePct = percentage;
	if (thePct > 100) thePct = 100;
	update(getGraphics());
    }

    //----------------------------------------------------
    // Change the percentage that is displayed
    //----------------------------------------------------

    public double get_percent () {
        return thePct;
    }

     

    //----------------------------------------------------
    // Repaint in response to a change in the percentage
    //----------------------------------------------------
    

    public void update (Graphics g) {

	int w = 2*theRadius;
	int s1 = (int) (360 * theOldPct / 100);
	int s2 = (int) (360 * thePct / 100);

	if (theOldPct > thePct) {
	    g.setColor(theColor2);
	    g.fillArc(0, 0, w, w, s2, s1-s2);
	}
	else if (theOldPct < thePct) {
	    g.setColor(theColor1);
	    g.fillArc(0, 0, w, w, s1, s2-s1);
	}

    }
	    


    //----------------------------------------------------
    // Paint the chart from scratch
    //----------------------------------------------------

    public void paint (Graphics g) {
	int   w;
	int   s1;

	w= 2*theRadius;
	s1 = (int)(360d * thePct / 100d);
	
	g.setColor (theColor2);
	g.fillOval(0, 0, w, w);

	g.setColor(theColor1);
	g.fillArc(0, 0, w, w, 0, s1);

    }

}



