/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hamlet;

import java.awt.*;

/** 
  * Provides a popup help dialog box.  Help text is displayed along
  * with a <tt>Close</tt> button that removes the dialog from the
  * display. 
  */

public class HelpDialog extends Frame {

    private Button stopButton;
    private TextArea text;

    /**
      * Creates an empty help dialog with the specified title.  The
      * initial size is 50 columns wide and 20 rows high.  The
      * dialog is not displayed; this is done by calling the
      * <tt>show</tt> method.
      * @param title  The title of the help dialog
      */

    public HelpDialog (String title) {
	setTitle(title);
	setLayout(new BorderLayout(0,0));
	Panel stop = new Panel();
	stop.setLayout(new FlowLayout(FlowLayout.LEFT));
	stopButton = new Button("Close");
	stop.add(stopButton);
	add("North", stop);
	text = new TextArea(20,50);
	text.setEditable(false);
	add("Center", text);
	pack();
    }



    /**
      * Sets the text that is displayed in the help dialog.
      */

    public void setText (String str) {
	text.setText(str);
    }
	


    /**
      * Looks for a click on the <tt>Close</tt> button, at which
      * point the dialog is removed from the display.
      * @param theEvent    Event that triggered the action
      * @param arg         Argument associated with the event
      * @return            True if event handled, false otherwise
      */

    public boolean action (Event theEvent, Object arg) {

	if (theEvent.target instanceof Button) {

	    if (theEvent.id != Event.ACTION_EVENT)
		return false;

	    if (theEvent.target.equals(stopButton)) {
		show(false);
		return(true);
	    }

	}

	return(false);

    }


}
