/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hamlet;

import java.awt.*;

public class NumLabel {

    public double dvalue;
    private String units;
    private Component component;
    private int theX;
    private int theY;
    private int theHeight = 0;
    private int theWidth = 0;
    private int position = Label.LEFT;

    public NumLabel (Component comp, double init, int x, int y,
		     int pos) {
	component = comp;
	dvalue = init;
	units = null;
	theX = x;
	theY = y;
	position = pos;
    }


    public NumLabel (Component comp, double init, String unts,
		     int x, int y, int pos) {
	component = comp;
	dvalue = init;
	units = unts;
	theX = x;
	theY = y;
	position = pos;
    }


    public void setNumber (double label) {
	dvalue = label;
	paint(component.getGraphics());
    }
	    

    public void move (int x, int y) {
	theX = x;
	theY = y;
    }
    

    public int height () {
	return(theHeight);
    }

    public int width () {
	return(theWidth);
    }


    public void paint (Graphics g) {

	if (g == null) return;

	// Erase the current label

	g.setColor(component.getBackground());

	if (position == Label.LEFT) 
	    g.fillRect(theX, theY, theWidth, theHeight);
	else
	    g.fillRect(theX - theWidth, theY, theWidth, theHeight);


	// Create the new label
	
	String label = Double.toString(dvalue);
	if (units != null) label = label + " " + units;

	// Determine its size
	
	FontMetrics fm = component.getFontMetrics(component.getFont());
	theWidth = fm.stringWidth(label);
	theHeight = fm.getAscent() + fm.getDescent();

	// Display the label

	g.setColor(component.getForeground());
	g.setFont( component.getFont() );
	if (position == Label.LEFT)
	    g.drawString(label, theX, theY+fm.getAscent());
	else
	    g.drawString(label, theX - theWidth, theY+fm.getAscent());
	    
    }	

}
