/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hamlet;

import java.awt.*;

public class NumericLabel extends Canvas {

    private String legend;
    private int legendWidth;
    
    private String units;
    private int unitsWidth;

    private double dvalue;
    private int alignment;

    private int maxWidth = 0;

    private FontMetrics metrics;

    public NumericLabel (double init, int align) {
	legend = null;
	alignment = align;
	dvalue = init;
	units = null;
    }

    public NumericLabel (String leg, double init, int align) {
	if (leg != null) legend = leg + " ";
	else legend = null;
	alignment = align;
	dvalue = init;
	units = null;
    }


    public NumericLabel (String leg, double init, int align, String unts) {
	if (leg != null) legend = leg + " ";
	else legend = null;
	alignment = align;
	dvalue = init;
	if (unts != null) units = " " + unts;
	else units = null;
    }


    public void preferredWidth (int width) {
	maxWidth = width;
    }

    public void setNumber (double label) {
	dvalue = label;
	Graphics g = getGraphics();
	if (g != null) update(g);
    }


    public void addNotify () {
	super.addNotify();
	metrics = getFontMetrics(getFont());
	legendWidth = 0;
	unitsWidth = 0;
	if (legend != null)
	    legendWidth = metrics.stringWidth(legend);
	if (units != null)
	    unitsWidth = metrics.stringWidth(units);
    }


    public void update (Graphics g) {

	if (alignment == Label.LEFT)
	    g.clipRect(legendWidth, 0,
			size().width-legendWidth, size().height);

	else if (alignment == Label.RIGHT)
	    g.clipRect(0, 0,
		       size().width-unitsWidth, size().height);

	super.update(g);

    }


    public void paint (Graphics g) {
	g.setColor(getForeground());
	String label = "";
	if (legend != null) label = legend;
	label = label + Double.toString(dvalue);
	if (units != null) label = label + units;
	int width = metrics.stringWidth(label);
	
	int x = 0;
	if (alignment == Label.CENTER)
	    x = (size().width - width) / 2;
	else if (alignment == Label.RIGHT)
	    x = (size().width - width);

	g.drawString(label,
		     x,
		     metrics.getAscent() +
		         (size().height - metrics.getHeight())/2);
    }


    public Dimension preferredSize () {

	Dimension d = new Dimension(0,0);

	if (maxWidth > 0)
	    d.width = metrics.stringWidth("0") * maxWidth;
	else
	    d.width =
	      Math.max(metrics.stringWidth(Double.toString(Double.MAX_VALUE)),
		       metrics.stringWidth(Double.toString(Double.MIN_VALUE)));

	if (legend != null) d.width += legendWidth;
	if (units != null) d.width += unitsWidth;

	d.height = metrics.getAscent()*2;
	return(d);
    }

}
