/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

//******************************************************************
// ScrollableNumbers.java - Contains a class that allows input
//                          of numbers with large and small
//                          incremental adjustment buttons,
//                          as well as direct input.
//
//                          Numbers are stored internally as double
//
// Author:  George Stebbins
//
//******************************************************************
 
package hamlet;

//----------------------------------------------------
// These are the needed inclusions
//----------------------------------------------------
 
import java.awt.*;



//******************************************************************
//******************************************************************
//
// ScrollableNumbers class
//
//******************************************************************
//******************************************************************

public class ScrollableNumbers extends Panel {

    // What component to notify when a button is pushed

    Component                 whoToNotify;

    double                    currentValue;

    
    // Increments
    
    double                    small;
    double                    large;

    
    // Limits

    double                    lowerLimit;
    double                    upperLimit;


    // Names of the buttons

    String                    largeLeftName = "<<";
    String                    smallLeftName = "<";
    String                    smallRightName = ">";
    String                    largeRightName = ">>";

    
    // Parts of the object
    
    private Button            largeLeft;
    private Button            smallLeft;
    private TextField         theText;
    private Button            smallRight;
    private Button            largeRight;


    // What type data do we hold
    
    private int               theType;
    
    private static int        INT = 0;
    private static int        FLOAT = 1;
    private static int        DOUBLE = 2;


    // Where does the label go in relationship to the number
    
    public final static int   LEFT = 0;
    public final static int   TOP = 1;
    public final static int   RIGHT = 2;
    public final static int   BOTTOM = 3;

    
    //----------------------------------------------------
    // Constructor(s)
    //----------------------------------------------------
    
    // Initialize all types
    
    private void init (String    initValue,
		       double    smallIncrement,
		       double    largeIncrement,
		       double    lLimit,
		       double    uLimit,
		       int       numCharacters,
		       int       type,
		       int       labelLoc,
		       String    inputLabel,
		       Component notifyThis) {
	
	Label    label;


	small = (double)smallIncrement;
	large = (double)largeIncrement;
	lowerLimit = lLimit;
	upperLimit = uLimit;
	whoToNotify = notifyThis;
	theType = type;
	
	largeLeft  = new Button (largeLeftName);
	smallLeft  = new Button (smallLeftName);
	theText    = new TextField (initValue, numCharacters);
	smallRight = new Button (smallRightName);
	largeRight = new Button (largeRightName);

	Panel controls = new Panel();
	controls.setLayout (new FlowLayout(FlowLayout.CENTER,0,0));
	if (largeIncrement > smallIncrement) controls.add (largeLeft);
	controls.add (smallLeft);
	controls.add (theText);
	controls.add (smallRight);
	if (largeIncrement > smallIncrement) controls.add (largeRight);

	
	switch (labelLoc) {
	case BOTTOM:
	    setLayout (new GridLayout(2,1,0,0));
	    add (controls);
	    if (inputLabel != null) {
		label = new Label (inputLabel, Label.CENTER);
		add (label);
	    }
	    break;
	    
	case TOP:
	    setLayout (new GridLayout(2,1,0,0));
	    if (inputLabel != null) {
		label = new Label (inputLabel, Label.CENTER);
		add (label);
	    }
	    add (controls);
	    break;
	    
	case RIGHT:
	    setLayout (new FlowLayout(FlowLayout.CENTER,0,0));
	    add (controls);
	    if (inputLabel != null) {
		label = new Label (inputLabel, Label.LEFT);
		add (label);
	    }
	    break;
	    
	case LEFT:
 	    default:
	    setLayout (new FlowLayout(FlowLayout.CENTER,0,0));
	    if (inputLabel != null) {
		label = new Label (inputLabel, Label.RIGHT);
		add (label);
	    }
	    add (controls);
	    break;

	}
	
	setValue(initValue);
    }
    

    // Integers 
    
    public ScrollableNumbers (int       initValue,
			      int       smallIncrement,
			      int       largeIncrement,
			      int       lLimit,
			      int       uLimit,
			      int       numCharacters,
			      int       labelLoc,
			      String    label,
			      Component notifyThis) {
	String sInitValue;

	sInitValue = String.valueOf(initValue);
	init (sInitValue, smallIncrement, largeIncrement,
	      (double)lLimit, (double)uLimit,
	      numCharacters, INT, labelLoc, label, notifyThis);
	currentValue = (double)initValue;
	
    }

    
    // floats
    
    public ScrollableNumbers (float     initValue,
			      float     smallIncrement,
			      float     largeIncrement,
			      float     lLimit,
			      float     uLimit,
			      int       numCharacters,
			      int       labelLoc,
			      String    label,
			      Component notifyThis) {
	String sInitValue;

	sInitValue = String.valueOf(initValue);
	init (sInitValue, smallIncrement, largeIncrement,
	      (double)lLimit, (double)uLimit,
	      numCharacters, FLOAT, labelLoc, label, notifyThis);
	currentValue = (double)initValue;
    }

    
    // Doubles
    
    public ScrollableNumbers (double    initValue,
			      double    smallIncrement,
			      double    largeIncrement,
			      double    lLimit,
			      double    uLimit,
			      int       numCharacters,
			      int       labelLoc,
			      String    label,
			      Component notifyThis) {
	String sInitValue;

	sInitValue = String.valueOf(initValue);
	init (sInitValue, smallIncrement, largeIncrement,
	      lLimit, uLimit,
	      numCharacters, DOUBLE, labelLoc, label, notifyThis);
	currentValue = (double)initValue;
    }


    //----------------------------------------------------
    // action to do when a button is pushed
    //----------------------------------------------------

    public boolean action (Event theEvent, Object argument) {

	double increment;
	Event evt = copyEvent(theEvent);

	
	// User has changed the text field
	
	if (theEvent.target instanceof TextField) {
	    try { currentValue = readText();}
  	      catch (NumberFormatException nfe) {
		  System.out.print('\007');
		  System.out.flush();
		  return(true);
	      }

	    if (currentValue < lowerLimit) {
		theText.setValue(lowerLimit);
		currentValue = lowerLimit;
		System.out.print('\007');
		System.out.flush();
	    }
	    else if (currentValue > upperLimit) {
		theText.setValue(upperLimit);
		currentValue = upperLimit;
		System.out.print('\007');
		System.out.flush();
	    }
	    whoToNotify.deliverEvent (evt);
	    return true;


	// User has pushed a button (<<, <, >, >>)
	    
	} else if (theEvent.target instanceof Button) {

	    // Decide which button
	    
	    String whichButton = argument.toString();
	    if (whichButton == largeLeftName) {
		increment = -large;
	    } else if (whichButton == smallLeftName) {
		increment = -small;
	    } else if (whichButton == smallRightName) {
		increment = small;
	    } else if (whichButton == largeRightName) {
		increment = large;
	    } else {
		System.out.println ("Unknown button!");
		return false;
	    }
	    

	    // Update the text field

	    currentValue += increment;
	    if (currentValue > upperLimit)
		currentValue = upperLimit;
	    if (currentValue < lowerLimit)
		currentValue = lowerLimit;

	    setValue(currentValue);
	    whoToNotify.deliverEvent (evt);
	    return true;


	// Handle it at a higher level
	    
	} else {
	    return false;
	}
    }


    //----------------------------------------------------
    // We want to send the same event to the component to
    // notify, but the object must be this ScrollableNumbers
    //----------------------------------------------------
    private Event copyEvent(Event source) {
      Event dest;
      dest = new Event(this, source.when, source.id, source.x,
		       source.y, source.key, source.modifiers, 
		       source.arg);
      return(dest);
    }

    //----------------------------------------------------
    // Read the Text field value
    //----------------------------------------------------
    
    public double readText () {
	
	double    value;
	Double    v;
	String    s;

	s = theText.getText();
	v = Double.valueOf(s);
	value = v.doubleValue();
	return value;
    }


    //----------------------------------------------------
    // Return the current value
    //----------------------------------------------------

    // As text
    
    public String getText () {
	return String.valueOf (currentValue);
    }


    // As int
    
    public int getIntText () {
	
	return (int)currentValue;
    }

    
    // As float
    
    public float getFloatText () {
	return (float)currentValue;
    }

    
    // As double
    
    public double getDoubleText () {
	return currentValue;
    }


    //----------------------------------------------------
    // Set the text field value
    //----------------------------------------------------

    // As a number to remain in the current format.

    public void setValue (double d) {
      if (theType == DOUBLE)
	setDoubleText(d);
      else if (theType == FLOAT)
	setFloatText(d);
      else
	setIntText(d);
    }


    // As text
    
    public void setText (String s) {
	
	theText.setText (s);
	theType = DOUBLE;
	try {
	    currentValue = Double.valueOf(s).doubleValue();
	} catch (NumberFormatException nfe) {
	    currentValue = 0.0d;
	}
    }

    
    // As Int
    
    public void setIntText (int i) {
	
	theText.setText (Integer.toString(i));
	theType = INT;
	currentValue = (double)i;
    }

    
    // As Float
    
    public void setFloatText (float f) {
	
	theText.setText (Float.toString(f));
	theType = FLOAT;
	currentValue = (double)f;
    }

    
    // As Double
    
    public void setDoubleText (double d) {
	
	theText.setText (Double.toString(d));
	theType = DOUBLE;
	currentValue = d;
    }


    //----------------------------------------------------
    // Set the text field limits
    //----------------------------------------------------

    public void setLimits (double lower, double upper) {
	lowerLimit = lower;
	upperLimit = upper;
	if (lower > upper) lower = upper;
    }


    //----------------------------------------------------
    // Set the text field increments
    //----------------------------------------------------

    public void setIncrements (double big) {
	large = big;
	small = large / 10D;
    }


    //----------------------------------------------------
    // Set the text field increments
    //----------------------------------------------------

    public void setIncrements () {
	large = Math.abs(upperLimit - lowerLimit) / 10D;
	small = large / 10D;
    }


    public void setEditable (boolean b) {
	theText.setEditable(b);        	
    }

}




