/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hamlet;

import java.applet.*;
import java.awt.*;
import java.io.*;

import hamlet.*;

/**
  * Creates an applet that displays a button on the screen.  When that
  * button is pushed, a message is sent to an <tt>AppFrame</tt>
  * displayed elsewhere
  * on the screen.  The class of that frame is specified by the
  * <tt>CLASSNAME</tt> parameter to the applet, and the message to send
  * is specified by the <tt>MESSAGE</tt> parameter.  The label of
  * the button is specified by by <tt>LABEL</tt> parameter.  For example
  * <pre>
  *   &ltapplet CODE="SendButton.class"&gt
  *     &ltPARAM NAME=CLASSNAME VALUE="BlockStacker.BlockStacker"&gt
  *     &ltPARAM NAME=MESSAGE VALUE="add"&gt
  *     &ltPARAM NAME=LABEL VALUE="Add Block"&gt
  *   &lt/applet&gt
  * </pre>
  * would send the message <tt>add</tt> to a frame of class
  * <tt>BlockStacker.BlockStacker</tt>.
  * <P>
  * The applet uses the static methods of the <tt>AppButton</tt> class
  * to communicate with frames of a particular class.
  * @author  Joseph L. Zachary (zachary@cs.utah.edu), University of Utah
  * @see AppButton
  * @see AppFrame
  */

public class SendButton extends Applet {
 
    private String className;
    private String message;
    private Button theButton;


    /**
      * Obtains the <tt>CLASSNAME</tt> and <tt>MESSAGE</tt> parameters
      * and creates the button which, when will clicked, will send the
      * message.
      */

    public void init() {

	className = getParameter("Classname");

	if (className == null) {
	    System.out.println("Exiting: class name is missing " + className);
	    return;
	}

	message = getParameter("Message");

	if (message == null) {
	    System.out.println("Exiting: message is missing " + message);
	    return;
	}

	String label = getParameter("Label");
        if (label == null) label = "Send";

	theButton = new Button(label);
	theButton.setBackground(Color.yellow);
	add(theButton);

    }


    /**
      * When the button is clicked, sends the message to the appropriate
      * frame.
     * @param theEvent    Event that triggered the action
     * @param arg         Argument associated with the event
     * @return            True if event handled, false otherwise
     */

    public boolean action (Event theEvent, Object arg) {
	if (theEvent.target instanceof Button) {
	    if (theEvent.id != Event.ACTION_EVENT)
		return(false);
	    if (theEvent.target.equals(theButton)) {
		AppButton.sendMessage(className, message);
		return(true);
	    }
	}
	return(false);
    }


}
