/****************************************************************************

   Copyright (C) 1995 University of Utah

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation, 675 Massachusetts Avenue, Cambridge, MA 02139 USA;
   either version 1, or (at your option) any later version.

   The University of Utah disclaims all warranties with regard to this
   software, including all implied warranties of merchantability and fitness.
   In no event shall the University of Utah be liable for any special, indirect
   or consequential damages or any damages whatsoever resulting from loss of
   use, data or profits, whether in an action of contract, negligence or other
   tortious action, arising out of or in connection with the use or performance
   of this software.

*****************************************************************************/

#include <X11/Xlib.h>

#include <stdio.h>


/* A Communicator object manages the communication of synthetic events
   to other X windows. */

class Communicator {

public:

    Communicator ();

    ~Communicator ();

    /* Send the message encoded into "file" to a window named "handle".
       If all is false, stop when a line containing <HEAD> is encountered. */

    void send (char *handle, FILE *file, int all);


private:

    /* Return the KeyCode for "c" and or the appropriate mask into "mask". */

    KeyCode charToKeycode (char c, unsigned &mask);


    /* Return the KeyCode for "keysym" and or the appropriate mask into
       "mask. */

    KeyCode keysymToKeycode (KeySym keysym, unsigned &mask);


    /* Send characters in "msg" to the specified "window". */

    int doSendString (char *msg, Window window);
    

    /* Find a subwindow of "base" with the proper event mask. */

    Window doXFindSubWindow (Window base, long mask);


    /* Build a synthetic keypress event. */

    void buildKeypressEvent(Window w, XEvent &event);
    


    Display *display;        // Current display

    int screen;              // Current screen

};


