/****************************************************************************

   Copyright (C) 1995 University of Utah

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation, 675 Massachusetts Avenue, Cambridge, MA 02139 USA;
   either version 1, or (at your option) any later version.

   The University of Utah disclaims all warranties with regard to this
   software, including all implied warranties of merchantability and fitness.
   In no event shall the University of Utah be liable for any special, indirect
   or consequential damages or any damages whatsoever resulting from loss of
   use, data or profits, whether in an action of contract, negligence or other
   tortious action, arising out of or in connection with the use or performance
   of this software.

*****************************************************************************/

#include <X11/Xlib.h>

const int MAXPATH = 1024;    // Maximum filename length
const int MAXHANDLE = 100;   // Maximum window handle length
const int MAXCACHE = 10;     // Maximum number of cached handles
const int TIMEOUT = 15*60;   // Timeout interval for handle mappings (seconds)


/* A Control object keeps track of Hamlet state information; namely, the
   mapping between handles and windows. */


class Control {

public:

    /* Initialize the Hamlet control database from the given file.
       The default is ~/.hamlet. */

    Control (char *filename = NULL);


    /* Save the control database to the file from whence it came. */

    ~Control ();


    /* Clear out the control database. */

    void reset ();
    

    /* Insert a new handle/window/confirmation mapping, or update an
       existing one. */

    void insert (char *handle, Window window, int confirm);


    /* Retrieve the window and confirmation associated with the
       handle and update the timestamp.  If window is None, there
       was no mapping. */

    void fetch (char *handle, Window &window, int &confirm);


    /* Remove the mapping information associated with "handle". */

    void remove (char *handle);


private:

    struct Entry {
	char handle[MAXHANDLE];   // Window handle
	Window window;            // Window id
	long time;                // Timestamp
	int confirm;              // Confirmation bit
    };

    int count;                    // Number of entries

    Entry entries[MAXCACHE];      // Most recent entries

    char filename[MAXPATH];       // Underlying filename

};

	
