/****************************************************************************

   Copyright (C) 1995 University of Utah

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation, 675 Massachusetts Avenue, Cambridge, MA 02139 USA;
   either version 1, or (at your option) any later version.

   The University of Utah disclaims all warranties with regard to this
   software, including all implied warranties of merchantability and fitness.
   In no event shall the University of Utah be liable for any special, indirect
   or consequential damages or any damages whatsoever resulting from loss of
   use, data or profits, whether in an action of contract, negligence or other
   tortious action, arising out of or in connection with the use or performance
   of this software.

*****************************************************************************/

#ifdef sgi
#define __CP_STRING_H__
#endif

#include "xdialog.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/WinUtil.h>
#include <Xm/TextF.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/FileSB.h>
#include <Xm/MainW.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleBG.h>
#include <Xm/RowColumn.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include "hamlet.bitmap"

static XtAppContext _xd_app;
static Widget _xd_toplevel;

static char *fallbackResources[] = {
    "XDialog*Title: Hamlet",
    NULL};

void ok_callback_1(Widget d, XtPointer c_d, XtPointer cbs);
void radio_callback_1(Widget d, XtPointer n,
		                XmToggleButtonCallbackStruct *cbs);
void text_callback_1(Widget d, XtPointer c_d, XtPointer cbs);
void map_callback(Widget d, XtPointer c_d, XtPointer cbs);
void msg_callback(Widget d, XtPointer c_d, XtPointer cbs);

int xdialogok::done = 0;
int xdialogradio::done = 0;
int xdialogradio::choice = 0;
int xdialogtext::done = 0;



xdialogok::xdialogok(char *message, char *button)
{
    char *progname[] = {"xdialog", NULL};
    XmString t;
    XColor color;
    Pixmap bitmap;
    Pixel fg, bg;
    int depth;
    Arg args[3];
   
    XmString ok = XmStringCreateSimple(button);
    int nr = 1;

    XtSetArg(args[0], XmNautoUnmanage, False);
    XtSetArg(args[1], XmNcancelLabelString, ok);
    XtSetArg(args[2], XmNdefaultPosition, False);
    if (!_xd_toplevel)
	{
	    _xd_toplevel = XtVaAppInitialize(&_xd_app, "XDialog", NULL, 0, &nr, progname, fallbackResources, NULL);
	}

    dialog = XmCreateWarningDialog(_xd_toplevel, "Error", args, 3);

    XtAddCallback(dialog, XmNmapCallback, map_callback, NULL);

    XtVaGetValues(dialog, XmNforeground, &fg, XmNbackground, &bg, XmNdepth,
		  &depth, NULL);

    
    bitmap = XCreatePixmapFromBitmapData(XtDisplay(_xd_toplevel), RootWindowOfScreen(XtScreen(dialog)), hamlet_bits, hamlet_width, hamlet_height, fg, bg, depth);
    XtVaSetValues(dialog, XmNsymbolPixmap, bitmap, NULL);
    
    XtAddCallback(dialog, XmNcancelCallback, ok_callback_1, NULL);
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    t = XmStringCreateLtoR(message, XmFONTLIST_DEFAULT_TAG);
    XtVaSetValues(dialog, XmNmessageString, t, 
		  XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON,
		  NULL);
    XmStringFree(t);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
};




long xdialogok::activate()
{
    XEvent event;
    done = 0;
    XtRealizeWidget(dialog);
    while (!done)
	{
	    if (XtAppPending(_xd_app))
		{
		    XtAppNextEvent(_xd_app, &event);
		    XtDispatchEvent(&event);
		}
	}
    XtUnrealizeWidget(_xd_toplevel);
    while (XtAppPending(_xd_app))
	{
	    XtAppNextEvent(_xd_app, &event);
	    XtDispatchEvent(&event);
	}
    return done;
}


void ok_callback_1(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    int *answer = (int *)client_data;
    XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct *)call_data;
    switch(cbs->reason) 
	{
	case XmCR_CANCEL:
	    xdialogok::done = 1;
	    XtPopdown(XtParent(dialog));
	    break;
	default:
	    return;
	}
}


xdialogradio::xdialogradio(char *message, char *choices[])
{
    char *progname[] = {"xdialog", NULL};
    XmString t;
    Arg args[2];
    int nr = 1;
    char **i;
    if (!_xd_toplevel)
	{
	    _xd_toplevel = XtVaAppInitialize(&_xd_app, "XDialog", NULL, 0, &nr, progname, fallbackResources, NULL);
	}
    XtSetArg(args[0], XmNautoUnmanage, False);
    XtSetArg(args[1], XmNdefaultPosition, False);
    
    dialog = XmCreateMessageDialog(_xd_toplevel, "Choose", args, 2);

    XtAddCallback(dialog, XmNmapCallback, map_callback, NULL);
      {
	int ct=0;
	Widget choice[100];
	
	radio = XmVaCreateSimpleRadioBox(dialog,  "Select", 0,
		     radio_callback_1, NULL);
	for (i = choices; *i; i++)
	    {
		choice[ct] = XtVaCreateManagedWidget(*i,
		    xmToggleButtonGadgetClass, radio, NULL);
		XtAddCallback(choice[ct], XmNvalueChangedCallback,
		    radio_callback_1, (void *)ct);
		ct++;
	    };
	XtManageChild(radio);
    }
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
    
    t = XmStringCreateLtoR(message, XmFONTLIST_DEFAULT_TAG);
    XtVaSetValues(dialog, XmNmessageString, t, XmNdefaultPosition, False, NULL);
    XmStringFree(t);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
}


int xdialogradio::activate()
{
    XEvent event;
    done = 0; choice = 1;
    XtRealizeWidget(dialog);
    while (!done)
	{
	    if (XtAppPending(_xd_app))
		{
		    XtAppNextEvent(_xd_app, &event);
		    XtDispatchEvent(&event);
		}
	}
    XtPopdown(XtParent(dialog));
    XtUnrealizeWidget(_xd_toplevel);
    while (XtAppPending(_xd_app))
	{
	    XtAppNextEvent(_xd_app, &event);
	    XtDispatchEvent(&event);
	}
    return done;
}

void radio_callback_1(Widget radio, XtPointer n,
		      XmToggleButtonCallbackStruct *call_data)
{
    xdialogradio::choice = int(n) + 1;
    xdialogradio::done = int(n) + 1;
}

xdialogtext::xdialogtext(char *message, char *filename)
{
    char *progname[] = {"xdialog", NULL};
    XmString t;
    int nr = 1;
    Arg args[9];
    char **i;
 
    if (!_xd_toplevel)
	{
	    _xd_toplevel = XtVaAppInitialize(&_xd_app, "XDialog", NULL, 0, &nr, progname, fallbackResources, NULL);
	}

    XtSetArg(args[0], XmNautoUnmanage, False);
    XtSetArg(args[1], XmNdefaultPosition, False);

    dialog = XmCreateMessageDialog(_xd_toplevel, "Choose", args, 2);
    XtAddCallback(dialog, XmNmapCallback, map_callback, NULL);
    
    XtAddCallback(dialog, XmNokCallback, text_callback_1, NULL);
    XtAddCallback(dialog, XmNcancelCallback, text_callback_1, NULL);
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));

    {
	struct stat statb;
	FILE *fp;
	char *text;
	Widget text_w;

	XtSetArg(args[0], XmNrows, 6);
	XtSetArg(args[2], XmNeditable, False);
	XtSetArg(args[3], XmNeditMode, XmMULTI_LINE_EDIT);
	XtSetArg(args[4], XmNwordWrap, True);
	XtSetArg(args[5], XmNscrollHorizontal, False);
	XtSetArg(args[6], XmNblinkRate, 0);
	XtSetArg(args[7], XmNautoShowCursorPosition, True);
	XtSetArg(args[8], XmNcursorPositionVisible, False);

	text_w = XmCreateScrolledText(dialog, "text_w", args, 9);
		
	if (!(fp = fopen(filename, "r")))
	    {
		fprintf(stderr, "Error reading file %s\n", filename); 
		return;
	    }
	
	stat(filename, &statb);
	
	if (!(text = XtMalloc((unsigned)(statb.st_size+1))))
	    {
		fprintf(stderr, "Can't alloc enough space for file contents\n");
		fclose(fp);
		exit(0);
	    }
	if (!(fread(text, sizeof(char), statb.st_size+1, fp)))
	    fprintf(stderr, "Warning: may not have read entire file!\n");
	
	text[statb.st_size] = 0;
	XmTextSetString(text_w, text);
	XtFree(text);
	XtManageChild(text_w);
    }
    
    t = XmStringCreateLtoR(message, XmFONTLIST_DEFAULT_TAG);
    XtVaSetValues(dialog, XmNmessageString, t, NULL);
    XmStringFree(t);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
}

void xdialogtext::manageCancel() {
    XtManageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
}



int xdialogtext::activate()
{
    XEvent event;
    done = 0;
    XtRealizeWidget(dialog);
    while (!done)
	{
	    if (XtAppPending(_xd_app))
		{
		    XtAppNextEvent(_xd_app, &event);
		    XtDispatchEvent(&event);
		}
	}
    XtUnrealizeWidget(_xd_toplevel);
    while (XtAppPending(_xd_app))
	{
	    XtAppNextEvent(_xd_app, &event);
	    XtDispatchEvent(&event);
	}
    return done;
}

void text_callback_1(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    int *answer = (int *)client_data;
    XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct *)call_data;
    switch(cbs->reason) 
	{
	case XmCR_OK:
	    xdialogtext::done = 1;
	    XtPopdown(XtParent(dialog));
	    break;
	case XmCR_CANCEL:
	    xdialogtext::done = -1;
	    XtPopdown(XtParent(dialog));
	    break;
	default:
	    return;
	}
}


xdialogtar::xdialogtar(char *message, char *filename, int strip)
{
    FILE *fp, *fp2;
    char fname[12] = "/tmp/temp.";
    int done = 0;
    
    fname[10] = 48;
    fname[11] = 0;

    while (!done)
	{
	    while ((fp = fopen(fname, "r")) != NULL)
		fname[10]++;
	    
	    if ((fp = fopen(fname, "w")) == NULL)
		fname[10]++;
	    else
		done = 1;
	}
    
    char *command = new char[strlen(filename) + strlen(fname) + 100];

    if (command) {
	sprintf(command, "tail +%d %s | tar -tf - 2> /dev/null > %s",
		strip, filename, fname);
	system(command);
	xdt = new xdialogtext(message, fname);
	sprintf(command, "rm %s", fname);
	system(command);
	delete command;
    }
    
}

int xdialogtar::activate()
{
    xdt->manageCancel();
    return xdt->activate();
}

void map_callback(Widget d, XtPointer client_data, XtPointer call_data)
{
    unsigned long int a, b;
    static int getx, gety;
    int junk, junk2;
    unsigned int junk3;
    Dimension h, w;

    XtVaGetValues(d, XmNwidth, &w,
		  XmNheight, &h, NULL);
    
    XQueryPointer(XtDisplay(d), DefaultRootWindow(XtDisplay(d)),
		  &a, &b, &getx, &gety, &junk, &junk2, &junk3); 

    if ((getx + w/2 + 1) >= WidthOfScreen (XtScreen(d))) getx = 0;
    if ((gety + h/2 + 1) >= HeightOfScreen (XtScreen(d))) gety = 0;

    XtVaSetValues(d, XmNx, getx-w/2, XmNy, gety-h/2, NULL);
}

xdialogradiotext::xdialogradiotext(char *msg, char *choices[], char *msg2) :
    xdialogradio(msg, choices){
    message2 = msg2;
};

int xdialogradiotext::activate()
{
    XEvent event;
    Dimension h, w;
    XmString t;
    int done2;
    done = 0; choice = 1;
    XtRealizeWidget(dialog);
    while (!done)
	{
	    if (XtAppPending(_xd_app))
		{
		    XtAppNextEvent(_xd_app, &event);
		    XtDispatchEvent(&event);
		}
	}
    XtVaGetValues(dialog, XmNwidth, &w, XmNheight, &h, NULL);
    XtUnrealizeWidget(radio);

    t = XmStringCreateLtoR(message2, XmFONTLIST_DEFAULT_TAG);
    XtVaSetValues(XmMessageBoxGetChild(dialog, XmDIALOG_MESSAGE_LABEL), XmNlabelString, t);
    XtVaSetValues(dialog, XmNwidth, w, XmNheight, h, NULL);

    XmStringFree(t);
    done2 = 0;
    while (!done2)
	{
	    if (XtAppPending(_xd_app))
		{
		    XtAppNextEvent(_xd_app, &event);
		    XtDispatchEvent(&event);
		}
	    else
		done2 = 1;
	}
    return done;
}
    


xdialogradiotext::~xdialogradiotext()
{    
    XEvent event;
    XtPopdown(XtParent(dialog));
    XtUnrealizeWidget(_xd_toplevel);
    while (XtAppPending(_xd_app))
	{
	    XtAppNextEvent(_xd_app, &event);
	    XtDispatchEvent(&event);
	}
}


xdialogmsg::xdialogmsg(char *msg): xdialogok(msg, "")
{
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
}

long xdialogmsg::activate()
{
    XtRealizeWidget(dialog);
    Window targetWin=None;
    Display *display = XtDisplay(dialog);
    Window root = RootWindowOfScreen(XtScreen(_xd_toplevel));
    Cursor cursor = XCreateFontCursor(display, XC_crosshair);
    int status = XGrabPointer(display, root, False,
			      ButtonPressMask|ButtonReleaseMask, GrabModeSync,
			      GrabModeAsync, root, cursor, CurrentTime);
    XEvent event;
    int done = 0;

    if (status != GrabSuccess) return (None);


    int buttons = 0;
    int gotevent;
    while ((targetWin == None) || (buttons != 0)) {
	XAllowEvents(display, SyncPointer, CurrentTime);
	XtAppNextEvent(_xd_app, &event);
	XtDispatchEvent(&event);
	switch (event.type) {
	case ButtonPress:
	    if (targetWin == None) {
		targetWin = event.xbutton.subwindow;
		if (targetWin == None) targetWin = root;
	    }
	    buttons++;
	    break;
	case ButtonRelease:
	    if (buttons > 0) buttons--;
	    break;
      	}

    }

    XUngrabPointer(display, CurrentTime);
    XFreeCursor(display, cursor);
    XFlush(display);

    if (targetWin != None) {
	Window root;
	int x;
	unsigned int y;
	if (XGetGeometry(display, targetWin, &root, &x, &x,
			 &y, &y, &y, &y) && (targetWin != root))
            targetWin = XmuClientWindow(display, targetWin);
    }
    XtUnrealizeWidget(_xd_toplevel);
    while (XtAppPending(_xd_app))
	{
	    XtAppNextEvent(_xd_app, &event);
	    XtDispatchEvent(&event);
	}
    return(targetWin);
}

